package com.gycsabesz.huntitdown.ai
{
	import com.gycsabesz.huntitdown.controller.plane.AbstractPlaneController;
	import com.gycsabesz.huntitdown.controller.plane.EnemyPlaneController;
	import com.gycsabesz.huntitdown.controller.plane.PlaneController;
	import com.gycsabesz.huntitdown.model.GameModel;
	import com.gycsabesz.huntitdown.model.plane.PlaneModel;
	import com.gycsabesz.huntitdown.model.plane.PlaneModelImpl;
	import com.gycsabesz.huntitdown.model.weaponry.gun.MachineGunModel;
	import com.gycsabesz.huntitdown.model.weaponry.gun.RocketLauncherModel;
	import com.gycsabesz.huntitdown.view.GameView;
	import com.gycsabesz.huntitdown.view.planes.Plane;
	import com.gycsabesz.huntitdown.view.planes.Plane02;
	import com.gycsabesz.huntitdown.view.planes.PlaneTa152C;
	
	import flash.display.MovieClip;
	import flash.events.Event;

	/**
	 * <p>
	 * Computer AI, which uses the specified plane controller to controll the enemy plane.
	 * <p>
	 * Instance of this class will be created for each enemy planes.  
	 */
	public class EnemyPlaneAI
	{
		protected var target:PlaneController;
		protected var planeController:AbstractPlaneController;
		protected var gameView:GameView;
		
		/**
		 * Create a new AI for the specified enemy plane controller.
		 */
		public function EnemyPlaneAI(gameView:GameView, target:PlaneController, planeController:EnemyPlaneController)
		{			
			this.target = target;
			this.planeController = planeController;
			this.gameView = gameView;
			
			// actualize the view
			planeController.getView().addEventListener(Event.ENTER_FRAME, planeView_enterFrameHandler);
			planeController.getView().x = gameView.playground.x + gameView.playground.width + planeController.getView().width;
			planeController.getView().y = Math.random() * (gameView.playground.y + gameView.playground.height);
			
			planeController.setDirection(-1);
			planeController.getMachineGunController1().getModel().setInfinite(true);
			GameModel.getInstance().addPlane(planeController);

			gameView.gameObjectsContainer.addChild(planeController.getView());
			
			planeController.getView().addEventListener(Event.REMOVED_FROM_STAGE, planeView_removedHandler);
		}
		
		protected function planeView_removedHandler(e:Event):void{
			planeController.getView().removeEventListener(Event.ENTER_FRAME, planeView_enterFrameHandler);
			planeController.getView().removeEventListener(Event.REMOVED_FROM_STAGE, planeView_removedHandler);
		
			GameModel.getInstance().removePlane(planeController);
		}
		
		protected var firing:Boolean = false;
		
		/**
		 * <p>
		 * Callback method for ENTER_FRAME event.
		 * <p>
		 * Tries to approach the target plane. Also starts to fire when the target and the enemy plane are in the same position vertically.
		 */
		protected function planeView_enterFrameHandler(e:Event):void{
			planeController.moveForwardPlane();
			
			if (target.getView().x > planeController.getView().x || !target.getModel().isActive()){
				if (firing){
					planeController.stopFireMachineGun();
					firing = false;
				}	
				planeController.slowDownPlaneVertical(); 
			} else {
				if (Math.abs(target.getView().y - planeController.getView().y) < 60){
					if (!firing){
						planeController.startFireMachineGun();
						firing = true;
					}
					
					planeController.fireMachineGun();
					if (planeController.getView().x < gameView.playground.width - 100){
						if (planeController.getRocketLauncherController1().getModel().hasAmmo()){
							planeController.launchRocketLauncher1();
						}
					}
				} else {
					if (firing){
						planeController.stopFireMachineGun();
						firing = false;
					}			
				}		
				
				if (Math.abs(target.getView().y - planeController.getView().y) < 10){
					planeController.slowDownPlaneVertical(); 
					return;
				}
				
				if (target.getView().y > planeController.getView().y){
					planeController.moveDownPlane();
				} else if (target.getView().y < planeController.getView().y){
					planeController.moveUpPlane();
				}
			}
		}
	}
}