package com.gycsabesz.huntitdown.controller
{
	import com.gycsabesz.huntitdown.ai.EnemyPlaneAI;
	import com.gycsabesz.huntitdown.controller.item.ItemArmorGenerator;
	import com.gycsabesz.huntitdown.controller.item.ItemLifeGenerator;
	import com.gycsabesz.huntitdown.controller.item.ItemRocketGenerator;
	import com.gycsabesz.huntitdown.controller.item.ItemScoreGenerator;
	import com.gycsabesz.huntitdown.controller.level.Level1Controller;
	import com.gycsabesz.huntitdown.controller.plane.AbstractPlaneController;
	import com.gycsabesz.huntitdown.controller.plane.PlaneController;
	import com.gycsabesz.huntitdown.event.GameEvent;
	import com.gycsabesz.huntitdown.model.GameModel;
	import com.gycsabesz.huntitdown.model.plane.PlaneModelImpl;
	import com.gycsabesz.huntitdown.model.plane.PlayerPlaneModel;
	import com.gycsabesz.huntitdown.model.weaponry.gun.MachineGunModel;
	import com.gycsabesz.huntitdown.view.GameView;
	import com.gycsabesz.huntitdown.view.hud.ScoreText;
	import com.gycsabesz.huntitdown.view.hud.life.LifeHud;
	import com.gycsabesz.huntitdown.view.planes.KeyboardControlledPlaneTa152C;
	import com.gycsabesz.huntitdown.view.planes.PlaneTa152C;
	
	import flash.events.EventDispatcher;

	public class GameController
	{
		protected var model:GameModel;
		protected var gameView:GameView;
		
		protected var planeController:PlaneController;
		protected var planeView:PlaneTa152C;
		protected var planeModel:PlayerPlaneModel; 
		
		protected var itemLifeGenerator:ItemLifeGenerator
		protected var itemArmorGenerator:ItemArmorGenerator
		protected var itemScoreGenerator:ItemScoreGenerator
		protected var itemRocketGenerator:ItemRocketGenerator
		
		public function GameController(model:GameModel, gameView:GameView)
		{
			this.model = model;
			this.gameView = gameView;
			
			this.model.addEventListener(GameEvent.GAME_DEACTIVATED, model_gameDeactivatedHandler);
			this.model.addEventListener(GameEvent.GAME_ACTIVATED, model_gameActivatedHandler);
			
			// create user plane and controller, etc...
			planeModel = new PlayerPlaneModel();
			planeController = new PlaneController(planeModel, gameView);
			planeView = new KeyboardControlledPlaneTa152C(planeController);
			
			planeController.setView(planeView);
			planeController.getMachineGunController1().getModel().setInfinite(true);
			
			model.addPlane(planeController);
					
			gameView.gameObjectsContainer.addChild(planeView);
			planeView.x =  gameView.playground.width / 2;
			planeView.y =  gameView.playground.height / 2;
			
			// create hud
			var hud:LifeHud = new LifeHud(planeModel);
			gameView.hudContainer.addChild(hud);
			hud.y = 10;
			hud.x = gameView.playground.width - 10;
			
			// create score display HUD element
			var scoreText:ScoreText = new ScoreText(planeModel);
			gameView.hudContainer.addChild(scoreText);
			scoreText.y = 10;
			scoreText.x = gameView.playground.width / 2;
			scoreText.refresh();
			
			// create item generators
			itemLifeGenerator = new ItemLifeGenerator(100, 150, gameView, planeController);
			itemArmorGenerator = new ItemArmorGenerator(30, 50, gameView, planeController);
			itemScoreGenerator = new ItemScoreGenerator(20, 30, gameView, planeController);
			itemRocketGenerator = new ItemRocketGenerator(10, 60, gameView, planeController);
			
			// plane color handler
			this.model.addEventListener(GameEvent.GAME_PLANECOLOR_CHANGED, gameModel_gamePlanecolorChangedHandler);
		}
		
		/**
		 * Returns the game model.
		 */
		public function getModel():GameModel{
			return this.model;
		}
		
		/**
		 * Stop the game.
		 */
		public function deactivateGame():void {
			model.gameDeactivated();
		}
		
		/**
		 * Listener of plane color changing.
		 */
		protected function gameModel_gamePlanecolorChangedHandler(e:GameEvent):void {
			if (model.getPlaneColor() != null){
				planeView.graphic.transform.colorTransform = model.getPlaneColor();
			}
		}
		
		/**
		 * Listener for the game start event. Starts with level one - creates controller 1st level.
		 */
		protected function model_gameActivatedHandler(e:GameEvent):void{
			planeModel.resetModel();
			new Level1Controller(gameView, planeController);
		}
		
		/**
		 * Listener for the game stop event. Remove all enemies from the game, and remove the game view.
		 */
		protected function model_gameDeactivatedHandler(e:GameEvent):void{
			// remove all enemies from the game
			var i:Number = 0;
			while(model.getPlanes().length > 1){
				if (model.getPlanes()[i] == planeController){
					i++;
				} else {
					model.getPlanes()[i].getModel().setLife(0);
				}
			}
			
			if (this.gameView.parent != null) {
				this.gameView.parent.removeChild(this.gameView);
			}
		}
	}
}