package com.gycsabesz.huntitdown.controller
{
	import com.gycsabesz.huntitdown.event.GameEvent;
	import com.gycsabesz.huntitdown.model.GameModel;
	import com.gycsabesz.huntitdown.model.MenuModel;
	import com.gycsabesz.huntitdown.view.GameView;
	import com.gycsabesz.huntitdown.view.menu.MenuContainer;
	import com.gycsabesz.huntitdown.view.menu.pages.PageAbout;
	import com.gycsabesz.huntitdown.view.menu.pages.PageInstructions;
	import com.gycsabesz.huntitdown.view.menu.pages.PageMainMenu;
	import com.gycsabesz.huntitdown.view.menu.pages.PageNewGame;
	
	import flash.display.MovieClip;
	import flash.geom.ColorTransform;

	public class MenuController
	{
		protected var view:MenuContainer;
		protected var model:MenuModel;
		
		protected var gameView:GameView;
		protected var gameController:GameController;
		
		/**
		 * Instantiates a new menu controller.
		 */
		public function MenuController(model:MenuModel){
			this.model = model;
		}
		
		/**
		 * Go to the main menu page.
		 */
		public function gotoMainMenu():void {
			model.setPage(MenuModel.PAGE_MAIN_MENU);
		}
		
		/**
		 * <p>Starts a new game from the menu.
		 * <p><b>color</b>: color of the plane.
		 */
		public function startNewGame(color:ColorTransform):void {
			model.setPage(MenuModel.PAGE_NONE);
			
			if (gameController == null) {
				gameView = new GameView();
				this.view.parent.addChild(gameView);
				gameController = new GameController(GameModel.getInstance(), gameView);
				gameView.controller = gameController;
				GameModel.getInstance().addEventListener(GameEvent.GAME_DEACTIVATED, gameModel_gameDeactivatedHandler);
			} else {
				this.view.parent.addChild(gameView);
			}
			
			GameModel.getInstance().gameActivated();
			GameModel.getInstance().setPlaneColor(color);
		}
		
		/**
		 * Listener for game deactivated event.
		 */
		protected function gameModel_gameDeactivatedHandler(e:GameEvent):void{
			if (GameModel.getInstance().isGameCompleted()){
				this.gotoGameCompleted();
			} else {
				this.gotoGameFailed();
			}
		}
		
		/**
		 * Show game completed page.
		 */
		public function gotoGameCompleted():void{
			model.setPage(MenuModel.PAGE_GAME_COMPLETED);
		}
		
		/**
		 * Show game failed page.
		 */
		public function gotoGameFailed():void{
			model.setPage(MenuModel.PAGE_GAME_FAILED);
		}
		
		/**
		 * Show start new game page.
		 */
		public function gotoStartNewGame():void {
			model.setPage(MenuModel.PAGE_NEW_GAME);
		}
		
		/**
		 * Show about page.
		 */
		public function gotoAbout():void {
			model.setPage(MenuModel.PAGE_ABOUT);
		}
		
		/**
		 * Show instructions page.
		 */
		public function gotoInstructions():void{
			model.setPage(MenuModel.PAGE_INSTRUCTIONS);
		}
	
		/**
		 * Set the view of the menu.
		 */
		public function setView(view:MenuContainer):void{
			this.view = view;
		}
	}
}