package com.gycsabesz.huntitdown.controller.level
{
	import com.gycsabesz.huntitdown.ai.EnemyPlaneAI;
	import com.gycsabesz.huntitdown.controller.plane.AbstractPlaneController;
	import com.gycsabesz.huntitdown.controller.plane.EnemyPlaneController;
	import com.gycsabesz.huntitdown.controller.plane.PlaneController;
	import com.gycsabesz.huntitdown.event.GameEvent;
	import com.gycsabesz.huntitdown.event.InfoTextLineEvent;
	import com.gycsabesz.huntitdown.model.GameModel;
	import com.gycsabesz.huntitdown.model.plane.PlaneModel;
	import com.gycsabesz.huntitdown.model.plane.enemy.Plane02Model;
	import com.gycsabesz.huntitdown.model.plane.enemy.Plane03Model;
	import com.gycsabesz.huntitdown.model.plane.enemy.Plane04Model;
	import com.gycsabesz.huntitdown.model.plane.enemy.Plane05Model;
	import com.gycsabesz.huntitdown.model.plane.enemy.Plane06Model;
	import com.gycsabesz.huntitdown.view.GameView;
	import com.gycsabesz.huntitdown.view.hud.InfoTextLine;
	import com.gycsabesz.huntitdown.view.planes.Plane;
	import com.gycsabesz.huntitdown.view.planes.Plane02;
	import com.gycsabesz.huntitdown.view.planes.Plane03;
	import com.gycsabesz.huntitdown.view.planes.Plane04;
	import com.gycsabesz.huntitdown.view.planes.Plane05;
	import com.gycsabesz.huntitdown.view.planes.Plane06;
	
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	import flash.utils.getDefinitionByName;

	/**
	 * Controlls the game levels.
	 */
	public class LevelController
	{
		/**
		 * Unique ID of plane 2
		 */
		protected static const PLANE_02:String = "Plane02";
		
		/**
		 * Unique ID of plane 3
		 */
		protected static const PLANE_03:String = "Plane03";
		
		/**
		 * Unique ID of plane 4
		 */
		protected static const PLANE_04:String = "Plane04";
		
		/**
		 * Unique ID of plane 5
		 */
		protected static const PLANE_05:String = "Plane05";
		
		/**
		 * Unique ID of plane 6
		 */
		protected static const PLANE_06:String = "Plane06";
		
		protected var gameView:GameView;
		protected var planeController:PlaneController;
		
		protected var levelLength:Number;
		protected var generationMinimumInterval:Number;
		protected var generationMaximumInterval:Number;
		
		protected var finishTimer:Timer;
		protected var generationTimer:Timer;
		
		protected var availablePlanes:Array;
		
		/**
		 * <p>Creates a new level controller with the specified parameters.
		 * 
		 * <p>
		 * <b>planeController</b>: controller of player's plane.
		 * <br><b>levelLength</b>: the length of this level in milliseconds.
		 * <br><b>generationMinimumInterval</b>: new enemy plane should be generated after this interval is elapsed.
		 * <br><b>generationMaximumInterval</b>: new enemy plane should be generated before this interval is elapsed.
		 * <br><b>availablePlanes</b>: list of the planes which available at this level
		 */
		public function LevelController(
			gameView:GameView, 
			planeController:PlaneController, 
			levelLength:Number,
			generationMinimumInterval:Number,
			generationMaximumInterval:Number,
			... availablePlanes
		)
		{
			this.gameView = gameView;
			this.planeController = planeController;
			
			this.levelLength = levelLength;
			this.generationMaximumInterval = generationMaximumInterval;
			this.generationMinimumInterval = generationMinimumInterval;
			
			this.availablePlanes = availablePlanes;
			
			startFinishTimer();
			
			startGenerationTimer();
			
			GameModel.getInstance().addEventListener(GameEvent.GAME_DEACTIVATED, gameModel_gameDeactivatedHandler);
		}
		
		/**
		 * Game is stopped. Stop the internal timers.
		 */
		protected function gameModel_gameDeactivatedHandler(e:GameEvent):void{
			if (generationTimer != null){
				generationTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, generationTimer_timerHandler);
				generationTimer.stop();
			}
			
			if (finishTimer != null) {
				finishTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, finishTimer_timerHandler);
				finishTimer.stop();
			}
		}
		
		/**
		 * Game is started. Start the internal timers.
		 */
		protected function startFinishTimer():void{
			if (finishTimer != null){
				finishTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, finishTimer_timerHandler);
				finishTimer.stop();
			}
			
			finishTimer = new Timer(levelLength, 1);
			finishTimer.addEventListener(TimerEvent.TIMER_COMPLETE, finishTimer_timerHandler);
			finishTimer.start();
		}
		
		/**
		 * Creates a new timer for enemy plane generation. After the timer is completed, a new enemy plane will be generated and placed to the stage.
		 */
		protected function startGenerationTimer():void{
			if (generationTimer != null){
				generationTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, generationTimer_timerHandler);
				generationTimer.stop();
			}
			
			// randomize a new delay
			var delay:Number = generationMinimumInterval + Math.random() * (generationMaximumInterval - generationMinimumInterval); 

			// create a new timer with the randomized delay
			generationTimer = new Timer(delay, 1);
			generationTimer.addEventListener(TimerEvent.TIMER_COMPLETE, generationTimer_timerHandler);
			generationTimer.start();
		}
		
		/**
		 * Timer_Complete event listener. Generates a new enemy plane.
		 */
		protected function generationTimer_timerHandler(e:TimerEvent):void{
			// randomize class name based on available planes
			var index:Number = Math.floor(Math.random() * availablePlanes.length);
			var plane:String = availablePlanes[index];
			
			var modelClass:Class;
			var viewClass:Class;
			
			switch(plane){
				case PLANE_02:
					modelClass = Plane02Model;
					viewClass = Plane02;
					break;
				case PLANE_03:
					modelClass = Plane03Model;
					viewClass = Plane03;
					break;
				case PLANE_04:
					modelClass = Plane04Model;
					viewClass = Plane04;
					break;
				case PLANE_05:
					modelClass = Plane05Model;
					viewClass = Plane05;
					break;
				case PLANE_06:
					modelClass = Plane06Model;
					viewClass = Plane06;
					break;
			}
			
			// create enemy plane mode, view and controller
			var model:PlaneModel = new modelClass() as PlaneModel; 
			var controller:EnemyPlaneController = new EnemyPlaneController(model, gameView);
			var view:Plane = new viewClass(controller) as Plane;

			controller.setView(view);
			
			// create AI
			new EnemyPlaneAI(gameView, planeController, controller);
			
			// start the next generation phase (waiting and generating)
			startGenerationTimer();
		}
		
		/**
		 * Level finished. Go to the next level or finish the game, if there is no next levle.
		 */
		protected function finishTimer_timerHandler(e:TimerEvent):void{
			if (generationTimer != null){
				generationTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, generationTimer_timerHandler);
				generationTimer.stop();
			}
			
			finishTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, finishTimer_timerHandler);
			
			var infoTextLine:InfoTextLine;
			
			if (hasNextLevel()){
				infoTextLine = new InfoTextLine("Level Completed!", "Get ready for next level!", "3", "2", "1", "GO");
				infoTextLine.addEventListener(InfoTextLineEvent.PLAYING_FINISHED, nextLevelInfoText_playingFinishedHandler);
			} else {
				GameModel.getInstance().gameCompleted();
				infoTextLine = new InfoTextLine("Game Completed!", "Congratulation Soldier!");
				infoTextLine.addEventListener(InfoTextLineEvent.PLAYING_FINISHED, gameCompletedInfoText_playingFinishedHandler);
			}
			
			gameView.hudContainer.addChild(infoTextLine);
			infoTextLine.x = gameView.playground.width / 2;
			infoTextLine.y = gameView.playground.height / 2;
		}
	
		protected function nextLevelInfoText_playingFinishedHandler(e: InfoTextLineEvent):void{
			e.target.removeEventListener(InfoTextLineEvent.PLAYING_FINISHED, nextLevelInfoText_playingFinishedHandler);
			initializeNextLevel();
		}
	
		protected function gameCompletedInfoText_playingFinishedHandler(e: InfoTextLineEvent):void{
			e.target.removeEventListener(InfoTextLineEvent.PLAYING_FINISHED, gameCompletedInfoText_playingFinishedHandler);
			GameModel.getInstance().gameDeactivated();
		}
		
		/**
		 * <p>Returns the level controller of the next level, or null, if there is no next level.
		 * 
		 * <p>Should be implemented in subclasses.
		 */ 
		protected function initializeNextLevel():LevelController{
			return null;
		}
		
		/**
		 * <p>Checks whether there is next level.
		 * 
		 * <p>Should be implemented in subclasses.
		 */
		protected function hasNextLevel():Boolean{
			return false;
		}
	}
}