package com.gycsabesz.huntitdown.controller.plane
{
	import com.gycsabesz.huntitdown.controller.weaponry.MachineGunController;
	import com.gycsabesz.huntitdown.controller.weaponry.RocketLauncherController;
	import com.gycsabesz.huntitdown.event.GameEvent;
	import com.gycsabesz.huntitdown.event.PlaneEvent;
	import com.gycsabesz.huntitdown.model.GameModel;
	import com.gycsabesz.huntitdown.model.plane.PlaneModel;
	import com.gycsabesz.huntitdown.model.weaponry.gun.MachineGunModel;
	import com.gycsabesz.huntitdown.model.weaponry.gun.RocketLauncherModel;
	import com.gycsabesz.huntitdown.view.GameView;
	import com.gycsabesz.huntitdown.view.planes.Plane;
	import com.gycsabesz.huntitdown.view.weaponry.gun.MachineGun;
	import com.gycsabesz.huntitdown.view.weaponry.gun.RocketLauncher;
	
	import flash.display.MovieClip;
	import flash.events.EventDispatcher;
	
	public class AbstractPlaneController
	{
		protected var planeModel:PlaneModel;
		protected var planeView:Plane;
		protected var gameView:GameView;
		protected var direction:Number = 1;
		
		/**
		 * Creates a new plane controller for the specified model and view.
		 */
		public function AbstractPlaneController(planeModel:PlaneModel, gameView:GameView)
		{
			this.planeModel = planeModel;
			this.gameView = gameView;
			
			(planeModel as EventDispatcher).addEventListener(PlaneEvent.ARMOR_TOTALLY_DESTROYED, planeModel_armorTotallyDestroyedHandler);
			(planeModel as EventDispatcher).addEventListener(PlaneEvent.CHANGED_LIFE, planeModel_changedLifeHandler);
		}
		
		/**
		 * Event listener which will be executed when the plane's armor is totally destroyed.
		 */
		protected function planeModel_armorTotallyDestroyedHandler(e:PlaneEvent):void{
			getModel().decreaseLife();
			if (getModel().getLife() > 0){
				getModel().setArmor(getModel().getMaxArmor());
			}
			
			planeDestoryed();
		}
		
		/**
		 * Listener for plane's changelife event.
		 */
		protected function planeModel_changedLifeHandler(e:PlaneEvent):void{
			if (planeModel.getLife() == 0){				
				planeHasNoMoreLife();
			}
		}
		
		/**
		 * Set view
		 */
		public function setView(planeView:Plane):void{
			this.planeView = planeView;
			
			setupMachineGun1();
			setupMachineGun2();
			setupRocketLauncher1();
		}
		
		/**
		 * Get view.
		 */
		public function getView():Plane{
			return planeView;
		}
		
		/**
		 * Set model
		 */
		public function getModel():PlaneModel{
			return planeModel;
		}
		
		/******************************************
		 * SPEED AND MOVEMENT RELATED CODE
		 *****************************************/
		
		protected var friction:Number = 0.8;
		protected var vx:Number = 0, vy:Number = 0;
		
		/**
		 * <p>Set the direction of the plane.
		 * <p><b>direction</b>: horizontal direction. 1 means left to right, -1 means right to left.
		 */
		public function setDirection(direction:Number):void{
			this.direction = direction;
			this.planeView.scaleX = this.direction;
			
			if (machineGunController1 != null){
				machineGunController1.setDirection(direction);
			}
			
			if (machineGunController2 != null){
				machineGunController2.setDirection(direction);
			}
			
			if (rocketLauncherController1 != null){
				rocketLauncherController1.setDirection(direction);
			}
		}
		
		/**
		 * Moves up the plane
		 */
		public function moveUpPlane():void{
			vy -= planeModel.getAcceleration();
			updatePlanePosition();
		}
		
		/**
		 * Moves down the plane
		 */
		public function moveDownPlane():void{
			vy += planeModel.getAcceleration();
			updatePlanePosition();
		} 
		
		/**
		 * Moves the plane backward according to the specified direction.
		 */
		public function moveBackwardPlane():void{
			vx -= direction * planeModel.getAcceleration() / 2;
			updatePlanePosition();
		} 
		
		/**
		 * Moves the plane forward according to the specified direction.
		 */		
		public function moveForwardPlane():void{
			vx += direction * planeModel.getAcceleration();
			updatePlanePosition();
		}
		
		/**
		 * Decrease the verical speed of the plane.
		 */
		public function slowDownPlaneVertical():void{
			vy *= friction;
			updatePlanePosition();
		}
		
		/**
		 * Decrease the horizontal speed of the plane.
		 */
		public function slowDownPlaneHorizontal():void{
			vx *= friction;
			updatePlanePosition();
		}
		
		protected function updatePlanePosition():void{
			// check top speed
			if (direction * vx > planeModel.getTopSpeed())
				vx = direction * planeModel.getTopSpeed();
			else if (direction * vx < -planeModel.getTopSpeed()/2)
				vx = -1 * direction * planeModel.getTopSpeed()/2;
			
			if (vy > planeModel.getTopSpeed())
				vy = planeModel.getTopSpeed();
			else if (vy < -planeModel.getTopSpeed())
				vy = -planeModel.getTopSpeed();			
			
			// rotate and scale the plane a bit
			planeView.rotation = direction * vy/4;
			planeView.scaleY = (planeModel.getTopSpeed() - Math.abs(vy))/(planeModel.getTopSpeed()*4) + 0.9;			
			
			// update position
			planeView.x += vx;
			planeView.y += vy;
			
			// plane collision detect
			var planes:Vector.<AbstractPlaneController> = GameModel.getInstance().getPlanes();
			
			for (var i:Number = 0; i < planes.length; i++){
				var plane1:AbstractPlaneController  = planes[i];
				
				if (!plane1.getModel().isActive()){
					continue;
				}
				
				for (var j:Number = 0; j < planes.length; j++){
					var plane2:AbstractPlaneController  = planes[j];
					
					if (plane2 == plane1){
						continue;
					}
					
					if (!plane2.getModel().isActive()){
						continue;
					}
				
					if (plane1.getView().hitTestObject(plane2.getView())){
						var armor1:Number = plane1.getModel().getArmor();
						var armor2:Number = plane2.getModel().getArmor();
						
						// decrease the armor of two planes
						plane1.getModel().decreaseArmor(armor2);
						plane2.getModel().decreaseArmor(armor1);
						
						// increase the score of two planes
						plane1.getModel().increaseScore(armor2);
						plane2.getModel().increaseScore(armor1);
					}
				}
			}
			
			checkActualPosition();	
		}
		
		/**
		 * <p>Will be called after the positiof of the plane is updated.
		 * <p>Should be implemented in subclasses.
		 */
		protected function checkActualPosition():void{
				
		}
		
		/**
		 * <p>Will be called when this plane is destroyed.
		 * <p>Should be implemented in subclasses.
		 */
		protected function planeDestoryed():void{
			
		}
		
		/**
		 * <p>Will be called when this plane has no life.
		 * <p>Should be implemented in subclasses.
		 */
		protected function planeHasNoMoreLife():void{
			
		}
		
		/**************************************
		 * FIRING FUNCTIONALITY OF THE PLANE
		 **************************************/
		
		protected var machineGunController1:MachineGunController;
		protected var machineGunController2:MachineGunController;	
		protected var rocketLauncherController1:RocketLauncherController;
		
		public function getMachineGunController1():MachineGunController{
			return machineGunController1;
		}
		
		public function getMachineGunController2():MachineGunController{
			return machineGunController2;
		}
		
		public function getRocketLauncherController1():RocketLauncherController{
			return rocketLauncherController1;
		}
		
		/**
		 * Attach a rocket launcher to this plane.
		 */
		protected function setupRocketLauncher1():void{
			rocketLauncherController1 = new RocketLauncherController(planeModel.getRocketLauncher1(), gameView);
			
			var rocketLauncherView:RocketLauncher = new RocketLauncher(rocketLauncherController1);
			
			rocketLauncherController1.setView(rocketLauncherView);
			rocketLauncherController1.setDirection(direction);
			
			planeView.addChild(rocketLauncherView);
			
			rocketLauncherView.x = planeView.getMarkerRocketLauncher().x;
			rocketLauncherView.y = planeView.getMarkerRocketLauncher().y;
			
			rocketLauncherView.refresh();
		}
		
		/**
		 * Attach the 1st machine gun to this plane.
		 */
		protected function setupMachineGun1():void{
			machineGunController1 = new MachineGunController(planeModel.getMachineGun1(), gameView);
			
			var machineGunView:MachineGun = new MachineGun(machineGunController1);
			
			machineGunController1.setView(machineGunView);
			machineGunController1.setDirection(direction);
			
			planeView.addChild(machineGunView);
			
			machineGunView.x = planeView.getMarkerMachineGun1().x;
			machineGunView.y = planeView.getMarkerMachineGun1().y;
		}
		
		/**
		 * Attach the 2nd machine gun to this plane.
		 */
		protected function setupMachineGun2():void{
			machineGunController2 = new MachineGunController(planeModel.getMachineGun2(), gameView);
			
			var machineGunView:MachineGun = new MachineGun(machineGunController2);
			
			machineGunController2.setView(machineGunView);
			machineGunController2.setDirection(direction);
			
			planeView.addChild(machineGunView);
			
			machineGunView.x = planeView.getMarkerMachineGun2().x;
			machineGunView.y = planeView.getMarkerMachineGun2().y;
		}		
		
		/**
		 * Launch a rocket with rocket launcher.
		 */
		public function launchRocketLauncher1():void{
			if (rocketLauncherController1 != null && planeModel.isActive()){
				rocketLauncherController1.startFire();
				rocketLauncherController1.fire();
				rocketLauncherController1.stopFire();
			}
		}
		
		/**
		 * Starts the firing with available machine guns.
		 */
		public function startFireMachineGun():void{
			if (machineGunController1 != null && planeModel.isActive()){
				machineGunController1.startFire();
			}
			
			if (machineGunController2 != null && planeModel.isActive()){
				machineGunController2.startFire();
			}
		}
		
		/**
		 * Fire with available machine guns.
		 */
		public function fireMachineGun():void{
			if (machineGunController1 != null && planeModel.isActive()){
				machineGunController1.fire();
			}
			if (machineGunController2 != null && planeModel.isActive()){
				machineGunController2.fire();
			}			
		}
		
		/**
		 * Stop firing with available machine guns.
		 */
		public function stopFireMachineGun():void{
			if (machineGunController1 != null){
				machineGunController1.stopFire();
			}
			if (machineGunController2 != null){
				machineGunController2.stopFire();
			}
		}
	}
}