package com.gycsabesz.huntitdown.controller.weaponry
{
	import com.gycsabesz.huntitdown.controller.plane.AbstractPlaneController;
	import com.gycsabesz.huntitdown.model.GameModel;
	import com.gycsabesz.huntitdown.model.weaponry.bullet.BulletModel;
	import com.gycsabesz.huntitdown.view.GameView;
	import com.gycsabesz.huntitdown.view.Playground;
	import com.gycsabesz.huntitdown.view.weaponry.bullet.Bullet;
	
	import flash.display.MovieClip;

	/**
	 * Contoller of any kind of bullets (e.g.: bullet of machine gun, rocket of rocket launcher, etc...)
	 */
	public class BulletController
	{
		protected var model:BulletModel;
		protected var gameView:GameView;
		protected var view:Bullet;
			
		/**
		 * Create a bullet controller for the specified model.
		 */
		public function BulletController(model:BulletModel, gameView:GameView)
		{
			this.model = model;
			this.gameView = gameView;
		}
		
		/**
		 * Get bullet model.
		 */
		public function getModel():BulletModel{
			return model;
		}
		
		/**
		 * Set bullet view.
		 */
		public function setView(view:Bullet):void{
			this.view = view;
		}
		
		/**
		 * <p>Move the bullet. Bullet speed will be calculated from data specified in the model.
		 * <p>Also checks if it hits something and handles the effects of it.
		 */
		public function moveBullet():void{
			view.x = view.x + model.getGun().getDirection() * model.getGun().getBulletSpeed();
			
			var planes:Vector.<AbstractPlaneController> = GameModel.getInstance().getPlanes();
			
			for (var i:Number = 0; i < planes.length; i++){
				var target:AbstractPlaneController  = planes[i];
				if (view.hitTestObject(target.getView())){
					if (target.getModel() != model.getGun().getPlaneModel() && target.getModel().isActive()){
						view.removeAndPlayHit();
						
						// decrease the target armor
						target.getModel().decreaseArmor(model.getGun().getHitPoint());
						
						// increase the score with damaged armor
						model.getGun().getPlaneModel().increaseScore(model.getGun().getHitPoint());
						
						// if the target fully damaged then increase the socre with the original armor point
						if (target.getModel().getArmor() <= 0){
							model.getGun().getPlaneModel().increaseScore(target.getModel().getMaxArmor());
						}
					}
				}
			}
			
			if (view.x > this.gameView.playground.width || view.x < 0){
				view.remove();
			}
		}
	}
}