package com.gycsabesz.huntitdown.model
{
	import com.gycsabesz.huntitdown.controller.plane.AbstractPlaneController;
	import com.gycsabesz.huntitdown.event.GameEvent;
	import com.gycsabesz.huntitdown.model.plane.PlaneModel;
	
	import flash.display.MovieClip;
	import flash.events.EventDispatcher;
	import flash.geom.ColorTransform;

	/**
	 * Holds information about the entire game.
	 */
	public class GameModel extends EventDispatcher
	{
		static private var inst:GameModel;
		
		protected var cfgPlaneColor:ColorTransform;
		
		protected var planes:Vector.<AbstractPlaneController> = new Vector.<AbstractPlaneController>();
		
		protected var _gameCompleted:Boolean = false;
		protected var _gameFailed:Boolean = true;
		protected var _gameActivated:Boolean = false;
		
		public function GameModel(){
		}
		
		/**
		 * Adds a plane to the game
		 */
		public function addPlane(controller: AbstractPlaneController):void{
			planes.push(controller);
		}
		
		/**
		 * Returns the list of the current planes.
		 */
		public function getPlanes():Vector.<AbstractPlaneController>{
			return planes;
		}
		
		/**
		 * Removes a plane from the game.
		 */
		public function removePlane(controller:AbstractPlaneController):void {
			var index:Number = planes.indexOf(controller)
			if (planes.indexOf(controller) > -1){
				planes.splice(index, 1);
			}
		}
		
		/**
		 * Set color of player's plane.
		 */
		public function setPlaneColor(color:ColorTransform):void{
			this.cfgPlaneColor = color;
			dispatchEvent(new GameEvent(GameEvent.GAME_PLANECOLOR_CHANGED));
		}
		
		/**
		 * Return the color of player's plane.
		 */
		public function getPlaneColor():ColorTransform{
			return this.cfgPlaneColor;
		}
		
		/**
		 * <p>Returns the instance of game model. 
		 * <p>GameModel is a singleton.
		 */
		static public function getInstance():GameModel{
			if (GameModel.inst == null){
				GameModel.inst = new GameModel();
			}
			return GameModel.inst;
		}
		
		/**
		 * Stops the game.
		 */
		public function gameDeactivated():void{
			var prevState:Boolean = _gameActivated;
			this._gameActivated = false;
			if (prevState != _gameActivated){
				dispatchEvent(new GameEvent(GameEvent.GAME_DEACTIVATED));
			}
		}
		
		/**
		 * Starts the game.
		 */
		public function gameActivated():void{
			this._gameCompleted = false;
			this._gameFailed = false;
			
			var prevState:Boolean = _gameActivated;
			this._gameActivated = true;
			if (prevState != _gameActivated){
				dispatchEvent(new GameEvent(GameEvent.GAME_ACTIVATED));
			}
		}
		
		/**
		 * Notify the model about the fact that the game is successfully completed.
		 */ 
		public function gameCompleted():void{
			this._gameCompleted = true;
			this._gameFailed = false;
			dispatchEvent(new GameEvent(GameEvent.GAME_COMPLETED));
		}
		
		/**
		 * Notify the model about the fact that the game is failed.
		 */
		public function gameFailed():void{
			this._gameCompleted = false;
			this._gameFailed = true;
			dispatchEvent(new GameEvent(GameEvent.GAME_FAILED));
		}
		
		/**
		 * Is the game active?
		 */
		public function isGameActive():Boolean {
			return !this._gameActivated;
		}
		
		/**
		 * Is the game completed?
		 */
		public function isGameCompleted():Boolean {
			return this._gameCompleted;
		}
		
		/**
		 * Is the game failed?
		 */	
		public function isGameFailed():Boolean {
			return this._gameFailed;
		}
	}
}