package com.gycsabesz.huntitdown.model.plane
{
	import com.gycsabesz.huntitdown.model.weaponry.gun.MachineGunModel;
	import com.gycsabesz.huntitdown.model.weaponry.gun.RocketLauncherModel;

	/**
	 * Interface for all plane models
	 */
	public interface PlaneModel
	{
		/**
		 * Returns the top speed of the plane.
		 */
		function getTopSpeed():Number;
		
		/**
		 * Sets the top speed of the plane.
		 */
		function setTopSpeed(topSpeed:Number):void;
		
		/**
		 * Returns the acceleration of the plane.
		 */
		function getAcceleration():Number;
		
		/**
		 * Sets the acceleration of the plane. Will be used to calculate the actual speed of the plane in every frame. (Suggested value is around 1.0)
		 */
		function setAcceleration(acceleration:Number):void;
		
		/**
		 * Set the gained scores of this plane.
		 */
		function setScore(score:Number):void;
		
		/**
		 * Increase the scores with the specified amunt.
		 */ 
		function increaseScore(additionalScore:Number):void;
		
		/**
		 * Returns the gained scores of this plane.
		 */
		function getScore():Number;
		
		/**
		 * Set life count of the plane.
		 */
		function setLife(lifeCount:Number):void;
		
		/**
		 * Increase the life count of the plane with 1.
		 */
		function increaseLife():void;
		
		/**
		 * Decrease the life count of the plane with 1.
		 */
		function decreaseLife():void;
		
		/**
		 * Returns the life count of the plane.
		 */
		function getLife():Number;

		/**
		 * Increase the maximum life count of the plane with 1.
		 */
		function increaseMaxLife():void;
		
		/**
		 * Sets the maximum life count of the plane.
		 */
		function setMaxLife(maxLife:Number):void;
		
		/**
		 * Returns the maximum life count of the plane.
		 */
		function getMaxLife():Number;
		
		/**
		 * Increase the armor strength of the plane with the specified value.
		 */
		function increaseArmor(additionalArmor:Number):void;
		
		/**
		 * Decrease the armor strength of the plane with the specified value.
		 */
		function decreaseArmor(minusArmor:Number):void;
		
		/**
		 * Set the armor strength of the plane for the specified value.
		 */
		function setArmor(armor:Number):void;
		
		/**
		 * Returns the armor strength of the plane.
		 */
		function getArmor():Number;
		
		/**
		 * Increase the maximum armor strength of the plane with the specified value.
		 */
		function increaseMaxArmor(additionalMaxArmor:Number):void;
		
		/**
		 * Set the maximum armor strength of the plane.
		 */
		function setMaxArmor(maxArmor:Number):void;
		
		/**
		 * Returns the armor strength of the plane.
		 */
		function getMaxArmor():Number;
		
		/**
		 * Set the model of 1st machine gun.
		 */
		function setMachineGun1(machineGun:MachineGunModel):void;
		
		/**
		 * Get the model of 1st machine gun.
		 */
		function getMachineGun1():MachineGunModel;
		
		/**
		 * Set the model of 2nd machine gun.
		 */
		function setMachineGun2(machineGun:MachineGunModel):void;
		
		/**
		 * Get the model of 2nd machine gun.
		 */
		function getMachineGun2():MachineGunModel;
		
		/**
		 * Set the model of 1st rocket laundher.
		 */
		function setRocketLauncher1(rocketLauncher:RocketLauncherModel):void;
		
		/**
		 * Get the model of 1st rocket laundher.
		 */
		function getRocketLauncher1():RocketLauncherModel;
		
		/**
		 * Set if this plane is active. Typically, a plane is inactive after a new life is started.
		 */
		function setActive(value:Boolean):void;
		
		/**
		 * Returns if this plane is active or not.
		 */
		function isActive():Boolean;
	}
}