package com.gycsabesz.huntitdown.view.menu
{
	import com.gycsabesz.huntitdown.controller.MenuController;
	import com.gycsabesz.huntitdown.event.MenuEvent;
	import com.gycsabesz.huntitdown.model.MenuModel;
	import com.gycsabesz.huntitdown.view.menu.bg.BackgroundImage;
	import com.gycsabesz.huntitdown.view.menu.pages.PageAbout;
	import com.gycsabesz.huntitdown.view.menu.pages.PageGameCompleted;
	import com.gycsabesz.huntitdown.view.menu.pages.PageGameFailed;
	import com.gycsabesz.huntitdown.view.menu.pages.PageInstructions;
	import com.gycsabesz.huntitdown.view.menu.pages.PageMainMenu;
	import com.gycsabesz.huntitdown.view.menu.pages.PageNewGame;
	import com.gycsabesz.huntitdown.view.menu.sound.BackgroundMusic;
	
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.geom.Point;
	import flash.sampler.pauseSampling;
	
	/**
	 * Menu of the application.
	 */
	public class MenuContainer extends MovieClip
	{	
		public var pageLeftTop:MovieClip;
		
		protected var backgroundMusic:BackgroundMusic;
		
		protected var backgroundImage:BackgroundImage;
		protected var backgroundEnabled:Boolean = true;
		protected var actualPage:MovieClip;
		
		protected var model:MenuModel;
		protected var controller:MenuController;
		
		public function MenuContainer(model:MenuModel, controller:MenuController){
			this.model = model;
			this.controller = controller;
			
			this.backgroundMusic = new BackgroundMusic();
			
			this.addEventListener(Event.ADDED_TO_STAGE, this_addedToStageHandler);
			model.addEventListener(MenuEvent.CHANGED_PAGE, menu_changedPageHanlder);
		}
		
		/**
		 * Called when a changed page event is dispatched.
		 */ 
		protected function menu_changedPageHanlder(e:Event):void{
			switch(model.getPage()){
			case MenuModel.PAGE_ABOUT:
				setPage(new PageAbout(model, controller));
				break;
			case MenuModel.PAGE_INSTRUCTIONS:
				setPage(new PageInstructions(model, controller));
				break;
			case MenuModel.PAGE_MAIN_MENU:
				setPage(new PageMainMenu(model, controller));
				break;
			case MenuModel.PAGE_NEW_GAME:
				setPage(new PageNewGame(model, controller));
				break;
			case MenuModel.PAGE_GAME_COMPLETED:
				setPage(new PageGameCompleted(model, controller));
				break;
			case MenuModel.PAGE_GAME_FAILED:
				setPage(new PageGameFailed(model, controller));
				break;
			case MenuModel.PAGE_NONE:
				removeActualPage();
				backgroundMusic.pause();
				break;
			}
		}
		
		/**
		 * Called when the menu container is added to the stage.
		 */
		protected function this_addedToStageHandler(e:Event):void{
			repositionBackground();
			backgroundMusic.play();
		}
		
		protected function repositionBackground():void{
			if (stage == null){
				return;
			}
			
			if (backgroundEnabled){
				if (backgroundImage == null){
					backgroundImage = new BackgroundImage(stage.stageWidth, stage.stageHeight);
					backgroundImage.x = this.globalToLocal(new Point(0,0)).x;
					backgroundImage.y = this.globalToLocal(new Point(0,0)).y;
				}
				this.addChildAt(backgroundImage, 0);
			} else {
				if (backgroundImage != null){
					this.removeChild(backgroundImage);
				}
			}
		}
		
		public function get background():Boolean {
			return backgroundEnabled;
		}
		
		public function set background(backgroundEnabled:Boolean):void{
			this.backgroundEnabled = backgroundEnabled;
			repositionBackground();
		}
		
		/**
		 * Remove the actual page.
		 */
		protected function removeActualPage():void{
			if (actualPage != null) {
				this.removeChild(actualPage);
			}
			actualPage = null;
		}
		
		/**
		 * Set the actual page.
		 */
		protected function setPage(page:MovieClip):void{
			if (parent != null){
				this.backgroundMusic.play();
			}
			
			removeActualPage();
			
			addChild(actualPage = page);
			
			actualPage.x = pageLeftTop.x;
			actualPage.y = pageLeftTop.y;
		}
	}
}